"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const fs = require("fs-extra");
const fs_extra_1 = require("fs-extra");
const path = require("path");
const Trie = require("cspell-trie-lib");
const gensequence_1 = require("gensequence");
const iterable_to_stream_1 = require("iterable-to-stream");
const zlib = require("zlib");
const UTF8 = 'utf8';
// eslint-disable-next-line @typescript-eslint/no-var-requires
const packageInfo = require('../package.json');
const version = packageInfo['version'];
function run(program, argv) {
    program.version(version);
    program
        .command('create <file.txt>')
        .option('-o, --output <file>', 'output file - defaults to stdout')
        .option('-l, --lower_case', 'output in lower case')
        .option('-b, --base <number>', 'Use base n for reference ids.  Defaults to 32. Common values are 10, 16, 32. Max of 36')
        .description('Generate a file for use with cspell')
        .action(async (filename, options) => {
        const { output: outputFile, lower_case: lowerCase = false, base = 32 } = options;
        notify('Create Trie', !!outputFile);
        const pOutputStream = createWriteStream(outputFile);
        notify(`Generating...`, !!outputFile);
        const lines = await fileToLines(filename);
        const toLower = lowerCase ? (a) => a.toLowerCase() : (a) => a;
        const wordsRx = lines
            .map(toLower)
            .map((a) => a.trim())
            .filter((a) => !!a);
        notify('Processing Trie');
        const trie = Trie.buildTrie(wordsRx);
        notify('Export Trie');
        const serialStream = iterable_to_stream_1.iterableToStream(Trie.serializeTrie(trie.root, base - 0 || 32));
        const outputStream = await pOutputStream;
        return new Promise((resolve) => {
            serialStream.pipe(outputStream).on('finish', () => resolve());
        });
    });
    program
        .command('reader <file.trie>')
        .option('-o, --output <file>', 'output file - defaults to stdout')
        .description('Read a cspell trie file and output the list of words.')
        .action(async (filename, options) => {
        const { output: outputFile } = options;
        notify('Reading Trie', !!outputFile);
        const pOutputStream = createWriteStream(outputFile);
        const lines = await fileToLines(filename);
        const root = Trie.importTrie(lines);
        const words = Trie.iteratorTrieWords(root);
        const outputStream = await pOutputStream;
        return new Promise((resolve) => {
            iterable_to_stream_1.iterableToStream(words.map((a) => a + '\n'))
                .pipe(outputStream)
                .on('finish', () => resolve());
        });
    });
    try {
        return program.parseAsync(argv);
    }
    catch (e) {
        return Promise.reject(e);
    }
}
exports.run = run;
async function fileToLines(filename) {
    const buffer = await fs.readFile(filename);
    const file = (filename.match(/\.gz$/) ? zlib.gunzipSync(buffer) : buffer).toString(UTF8);
    return gensequence_1.genSequence(file.split(/\r?\n/));
}
function createWriteStream(filename) {
    return !filename
        ? Promise.resolve(process.stdout)
        : fs_extra_1.mkdirp(path.dirname(filename)).then(() => fs.createWriteStream(filename));
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function notify(message, useStdOut = true) {
    if (useStdOut) {
        console.log(message);
    }
    else {
        console.error(message);
    }
}
//# sourceMappingURL=app.js.map